@@ -1,54 +1,50 @@
 from django.contrib import messages
-from django.contrib.auth import REDIRECT_FIELD_NAME
 from django.contrib.auth.decorators import login_required
 from django.contrib.sites.shortcuts import get_current_site
 from django.core.exceptions import PermissionDenied
 from django.core.validators import validate_email
 from django.forms import ValidationError
-from django.http import (
-    Http404,
-    HttpResponse,
-    HttpResponsePermanentRedirect,
-    HttpResponseRedirect,
-)
+from django.http import Http404, HttpResponse, HttpResponseRedirect
 from django.urls import reverse, reverse_lazy
 from django.utils.decorators import method_decorator
 from django.views.decorators.cache import never_cache
 from django.views.decorators.debug import sensitive_post_parameters
-from django.views.generic.base import TemplateResponseMixin, TemplateView, View
+from django.views.generic.base import TemplateView
 from django.views.generic.edit import FormView
 
 from allauth import app_settings as allauth_app_settings
-from allauth.account import app_settings, signals
+from allauth.account import app_settings
 from allauth.account.adapter import get_adapter
-from allauth.account.decorators import reauthentication_required
 from allauth.account.forms import (
     AddEmailForm,
     ChangePasswordForm,
+    ConfirmLoginCodeForm,
     LoginForm,
     ReauthenticateForm,
+    RequestLoginCodeForm,
     ResetPasswordForm,
     ResetPasswordKeyForm,
     SetPasswordForm,
     SignupForm,
     UserTokenForm,
 )
+from allauth.account.internal import flows
+from allauth.account.mixins import (
+    AjaxCapableProcessFormViewMixin,
+    CloseableSignupMixin,
+    LogoutFunctionalityMixin,
+    NextRedirectMixin,
+    RedirectAuthenticatedUserMixin,
+    _ajax_response,
+)
 from allauth.account.models import (
     EmailAddress,
     EmailConfirmation,
-    EmailConfirmationHMAC,
-)
-from allauth.account.reauthentication import (
-    record_authentication,
-    resume_request,
+    get_emailconfirmation_model,
 )
+from allauth.account.reauthentication import resume_request
 from allauth.account.utils import (
     complete_signup,
-    emit_email_changed,
-    get_login_redirect_url,
-    get_next_redirect_url,
-    logout_on_password_change,
-    passthrough_next_redirect_url,
     perform_login,
     send_email_confirmation,
     sync_user_email_addresses,
@@ -57,9 +53,9 @@ from allauth.account.utils import (
 )
 from allauth.core import ratelimit
 from allauth.core.exceptions import ImmediateHttpResponse
-from allauth.core.internal.http import redirect
+from allauth.core.internal.httpkit import redirect
 from allauth.decorators import rate_limit
-from allauth.utils import get_form_class, get_request_param
+from allauth.utils import get_form_class
 
 
 INTERNAL_RESET_SESSION_KEY = "_password_reset_key"
@@ -70,105 +66,26 @@ sensitive_post_parameters_m = method_decorator(
 )
 
 
-def _ajax_response(request, response, form=None, data=None):
-    adapter = get_adapter()
-    if adapter.is_ajax(request):
-        if isinstance(response, HttpResponseRedirect) or isinstance(
-            response, HttpResponsePermanentRedirect
-        ):
-            redirect_to = response["Location"]
-        else:
-            redirect_to = None
-        response = adapter.ajax_response(
-            request, response, form=form, data=data, redirect_to=redirect_to
-        )
-    return response
-
-
-class RedirectAuthenticatedUserMixin(object):
-    def dispatch(self, request, *args, **kwargs):
-        if request.user.is_authenticated and app_settings.AUTHENTICATED_LOGIN_REDIRECTS:
-            redirect_to = self.get_authenticated_redirect_url()
-            response = HttpResponseRedirect(redirect_to)
-            return _ajax_response(request, response)
-        else:
-            response = super(RedirectAuthenticatedUserMixin, self).dispatch(
-                request, *args, **kwargs
-            )
-        return response
-
-    def get_authenticated_redirect_url(self):
-        redirect_field_name = self.redirect_field_name
-        return get_login_redirect_url(
-            self.request,
-            url=self.get_success_url(),
-            redirect_field_name=redirect_field_name,
-        )
-
-
-class AjaxCapableProcessFormViewMixin(object):
-    def get(self, request, *args, **kwargs):
-        response = super(AjaxCapableProcessFormViewMixin, self).get(
-            request, *args, **kwargs
-        )
-        form = self.get_form()
-        return _ajax_response(
-            self.request, response, form=form, data=self._get_ajax_data_if()
-        )
-
-    def post(self, request, *args, **kwargs):
-        form_class = self.get_form_class()
-        form = self.get_form(form_class)
-        if form.is_valid():
-            response = self.form_valid(form)
-        else:
-            response = self.form_invalid(form)
-        return _ajax_response(
-            self.request, response, form=form, data=self._get_ajax_data_if()
-        )
-
-    def get_form(self, form_class=None):
-        form = getattr(self, "_cached_form", None)
-        if form is None:
-            form = super(AjaxCapableProcessFormViewMixin, self).get_form(form_class)
-            self._cached_form = form
-        return form
-
-    def _get_ajax_data_if(self):
-        return (
-            self.get_ajax_data()
-            if get_adapter(self.request).is_ajax(self.request)
-            else None
-        )
-
-    def get_ajax_data(self):
-        return None
-
-
-class LogoutFunctionalityMixin(object):
-    def logout(self):
-        adapter = get_adapter(self.request)
-        adapter.add_message(
-            self.request, messages.SUCCESS, "account/messages/logged_out.txt"
-        )
-        adapter.logout(self.request)
-
-
+@method_decorator(rate_limit(action="login"), name="dispatch")
 class LoginView(
-    RedirectAuthenticatedUserMixin, AjaxCapableProcessFormViewMixin, FormView
+    NextRedirectMixin,
+    RedirectAuthenticatedUserMixin,
+    AjaxCapableProcessFormViewMixin,
+    FormView,
 ):
     form_class = LoginForm
     template_name = "account/login." + app_settings.TEMPLATE_EXTENSION
     success_url = None
-    redirect_field_name = REDIRECT_FIELD_NAME
 
     @sensitive_post_parameters_m
     @method_decorator(never_cache)
     def dispatch(self, request, *args, **kwargs):
-        return super(LoginView, self).dispatch(request, *args, **kwargs)
+        if allauth_app_settings.SOCIALACCOUNT_ONLY and request.method != "GET":
+            raise PermissionDenied()
+        return super().dispatch(request, *args, **kwargs)
 
     def get_form_kwargs(self):
-        kwargs = super(LoginView, self).get_form_kwargs()
+        kwargs = super().get_form_kwargs()
         kwargs["request"] = self.request
         return kwargs
 
@@ -176,111 +93,75 @@ class LoginView(
         return get_form_class(app_settings.FORMS, "login", self.form_class)
 
     def form_valid(self, form):
-        success_url = self.get_success_url()
+        redirect_url = self.get_success_url()
         try:
-            return form.login(self.request, redirect_url=success_url)
+            return form.login(self.request, redirect_url=redirect_url)
         except ImmediateHttpResponse as e:
             return e.response
 
-    def get_success_url(self):
-        # Explicitly passed ?next= URL takes precedence
-        ret = (
-            get_next_redirect_url(self.request, self.redirect_field_name)
-            or self.success_url
-        )
-        return ret
-
     def get_context_data(self, **kwargs):
-        ret = super(LoginView, self).get_context_data(**kwargs)
-        signup_url = passthrough_next_redirect_url(
-            self.request, reverse("account_signup"), self.redirect_field_name
-        )
-        redirect_field_value = get_request_param(self.request, self.redirect_field_name)
+        ret = super().get_context_data(**kwargs)
+        signup_url = None
+        if not allauth_app_settings.SOCIALACCOUNT_ONLY:
+            signup_url = self.passthrough_next_url(reverse("account_signup"))
         site = get_current_site(self.request)
 
         ret.update(
             {
                 "signup_url": signup_url,
                 "site": site,
-                "redirect_field_name": self.redirect_field_name,
-                "redirect_field_value": redirect_field_value,
                 "SOCIALACCOUNT_ENABLED": allauth_app_settings.SOCIALACCOUNT_ENABLED,
+                "SOCIALACCOUNT_ONLY": allauth_app_settings.SOCIALACCOUNT_ONLY,
+                "LOGIN_BY_CODE_ENABLED": app_settings.LOGIN_BY_CODE_ENABLED,
             }
         )
+        if app_settings.LOGIN_BY_CODE_ENABLED:
+            request_login_code_url = self.passthrough_next_url(
+                reverse("account_request_login_code")
+            )
+            ret["request_login_code_url"] = request_login_code_url
         return ret
 
 
 login = LoginView.as_view()
 
 
-class CloseableSignupMixin(object):
-    template_name_signup_closed = (
-        "account/signup_closed." + app_settings.TEMPLATE_EXTENSION
-    )
-
-    def dispatch(self, request, *args, **kwargs):
-        try:
-            if not self.is_open():
-                return self.closed()
-        except ImmediateHttpResponse as e:
-            return e.response
-        return super(CloseableSignupMixin, self).dispatch(request, *args, **kwargs)
-
-    def is_open(self):
-        return get_adapter(self.request).is_open_for_signup(self.request)
-
-    def closed(self):
-        response_kwargs = {
-            "request": self.request,
-            "template": self.template_name_signup_closed,
-        }
-        return self.response_class(**response_kwargs)
-
-
 @method_decorator(rate_limit(action="signup"), name="dispatch")
 class SignupView(
     RedirectAuthenticatedUserMixin,
     CloseableSignupMixin,
+    NextRedirectMixin,
     AjaxCapableProcessFormViewMixin,
     FormView,
 ):
     template_name = "account/signup." + app_settings.TEMPLATE_EXTENSION
     form_class = SignupForm
-    redirect_field_name = REDIRECT_FIELD_NAME
-    success_url = None
 
     @sensitive_post_parameters_m
     @method_decorator(never_cache)
     def dispatch(self, request, *args, **kwargs):
-        return super(SignupView, self).dispatch(request, *args, **kwargs)
+        return super().dispatch(request, *args, **kwargs)
 
     def get_form_class(self):
         return get_form_class(app_settings.FORMS, "signup", self.form_class)
 
-    def get_success_url(self):
-        # Explicitly passed ?next= URL takes precedence
-        ret = (
-            get_next_redirect_url(self.request, self.redirect_field_name)
-            or self.success_url
-        )
-        return ret
-
     def form_valid(self, form):
         self.user, resp = form.try_save(self.request)
         if resp:
             return resp
         try:
+            redirect_url = self.get_success_url()
             return complete_signup(
                 self.request,
                 self.user,
-                app_settings.EMAIL_VERIFICATION,
-                self.get_success_url(),
+                email_verification=None,
+                success_url=redirect_url,
             )
         except ImmediateHttpResponse as e:
             return e.response
 
     def get_context_data(self, **kwargs):
-        ret = super(SignupView, self).get_context_data(**kwargs)
+        ret = super().get_context_data(**kwargs)
         form = ret["form"]
         email = self.request.session.get("account_verified_email")
         if email:
@@ -289,19 +170,14 @@ class SignupView(
                 email_keys.append("email2")
             for email_key in email_keys:
                 form.fields[email_key].initial = email
-        login_url = passthrough_next_redirect_url(
-            self.request, reverse("account_login"), self.redirect_field_name
-        )
-        redirect_field_name = self.redirect_field_name
+        login_url = self.passthrough_next_url(reverse("account_login"))
         site = get_current_site(self.request)
-        redirect_field_value = get_request_param(self.request, redirect_field_name)
         ret.update(
             {
                 "login_url": login_url,
-                "redirect_field_name": redirect_field_name,
-                "redirect_field_value": redirect_field_value,
                 "site": site,
                 "SOCIALACCOUNT_ENABLED": allauth_app_settings.SOCIALACCOUNT_ENABLED,
+                "SOCIALACCOUNT_ONLY": allauth_app_settings.SOCIALACCOUNT_ONLY,
             }
         )
         return ret
@@ -323,7 +199,7 @@ class SignupView(
 signup = SignupView.as_view()
 
 
-class ConfirmEmailView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
+class ConfirmEmailView(NextRedirectMixin, LogoutFunctionalityMixin, TemplateView):
     template_name = "account/email_confirm." + app_settings.TEMPLATE_EXTENSION
 
     def get(self, *args, **kwargs):
@@ -425,7 +301,7 @@ class ConfirmEmailView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
             return perform_login(
                 self.request,
                 user,
-                app_settings.EmailVerificationMethod.NONE,
+                email_verification=app_settings.EmailVerificationMethod.NONE,
                 # passed as callable, as this method
                 # depends on the authenticated state
                 redirect_url=self.get_redirect_url,
@@ -435,14 +311,10 @@ class ConfirmEmailView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
 
     def get_object(self, queryset=None):
         key = self.kwargs["key"]
-        emailconfirmation = EmailConfirmationHMAC.from_key(key)
+        model = get_emailconfirmation_model()
+        emailconfirmation = model.from_key(key)
         if not emailconfirmation:
-            if queryset is None:
-                queryset = self.get_queryset()
-            try:
-                emailconfirmation = queryset.get(key=key.lower())
-            except EmailConfirmation.DoesNotExist:
-                raise Http404()
+            raise Http404()
         return emailconfirmation
 
     def get_queryset(self):
@@ -460,7 +332,7 @@ class ConfirmEmailView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
         return ret
 
     def get_context_data(self, **kwargs):
-        ctx = kwargs
+        ctx = super().get_context_data(**kwargs)
         site = get_current_site(self.request)
         ctx.update(
             {
@@ -475,9 +347,12 @@ class ConfirmEmailView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
         return ctx
 
     def get_redirect_url(self):
-        return get_adapter(self.request).get_email_confirmation_redirect_url(
-            self.request
-        )
+        url = self.get_next_url()
+        if not url:
+            url = get_adapter(self.request).get_email_verification_redirect_url(
+                self.object.email_address,
+            )
+        return url
 
 
 confirm_email = ConfirmEmailView.as_view()
@@ -485,12 +360,6 @@ confirm_email = ConfirmEmailView.as_view()
 
 @method_decorator(login_required, name="dispatch")
 @method_decorator(rate_limit(action="manage_email"), name="dispatch")
-@method_decorator(
-    reauthentication_required(
-        allow_get=True, enabled=lambda request: app_settings.REAUTHENTICATION_REQUIRED
-    ),
-    name="dispatch",
-)
 class EmailView(AjaxCapableProcessFormViewMixin, FormView):
     template_name = (
         "account/email_change." if app_settings.CHANGE_EMAIL else "account/email."
@@ -503,7 +372,7 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
 
     def dispatch(self, request, *args, **kwargs):
         sync_user_email_addresses(request.user)
-        return super(EmailView, self).dispatch(request, *args, **kwargs)
+        return super().dispatch(request, *args, **kwargs)
 
     def get_form_kwargs(self):
         kwargs = super(EmailView, self).get_form_kwargs()
@@ -511,21 +380,8 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
         return kwargs
 
     def form_valid(self, form):
-        email_address = form.save(self.request)
-        adapter = get_adapter(self.request)
-        adapter.add_message(
-            self.request,
-            messages.INFO,
-            "account/messages/email_confirmation_sent.txt",
-            {"email": form.cleaned_data["email"]},
-        )
-        signals.email_added.send(
-            sender=self.request.user.__class__,
-            request=self.request,
-            user=self.request.user,
-            email_address=email_address,
-        )
-        return super(EmailView, self).form_valid(form)
+        flows.manage_email.add_email(self.request, form)
+        return super().form_valid(form)
 
     def post(self, request, *args, **kwargs):
         res = None
@@ -569,70 +425,13 @@ class EmailView(AjaxCapableProcessFormViewMixin, FormView):
     def _action_remove(self, request, *args, **kwargs):
         email_address = self._get_email_address(request)
         if email_address:
-            adapter = get_adapter()
-            if not adapter.can_delete_email(email_address):
-                adapter.add_message(
-                    request,
-                    messages.ERROR,
-                    "account/messages/cannot_delete_primary_email.txt",
-                    {"email": email_address.email},
-                )
-            else:
-                email_address.remove()
-                signals.email_removed.send(
-                    sender=request.user.__class__,
-                    request=request,
-                    user=request.user,
-                    email_address=email_address,
-                )
-                adapter.add_message(
-                    request,
-                    messages.SUCCESS,
-                    "account/messages/email_deleted.txt",
-                    {"email": email_address.email},
-                )
-                adapter.send_notification_mail(
-                    "account/email/email_deleted",
-                    request.user,
-                    {"deleted_email": email_address.email},
-                )
+            if flows.manage_email.delete_email(request, email_address):
                 return HttpResponseRedirect(self.get_success_url())
 
     def _action_primary(self, request, *args, **kwargs):
         email_address = self._get_email_address(request)
         if email_address:
-            # Not primary=True -- Slightly different variation, don't
-            # require verified unless moving from a verified
-            # address. Ignore constraint if previous primary email
-            # address is not verified.
-            if (
-                not email_address.verified
-                and EmailAddress.objects.filter(
-                    user=request.user, verified=True
-                ).exists()
-            ):
-                get_adapter().add_message(
-                    request,
-                    messages.ERROR,
-                    "account/messages/unverified_primary_email.txt",
-                )
-            else:
-                # Sending the old primary address to the signal
-                # adds a db query.
-                try:
-                    from_email_address = EmailAddress.objects.get(
-                        user=request.user, primary=True
-                    )
-                except EmailAddress.DoesNotExist:
-                    from_email_address = None
-                email_address.set_as_primary()
-                adapter = get_adapter()
-                adapter.add_message(
-                    request,
-                    messages.SUCCESS,
-                    "account/messages/primary_email_set.txt",
-                )
-                emit_email_changed(request, from_email_address, email_address)
+            if flows.manage_email.mark_as_primary(request, email_address):
                 return HttpResponseRedirect(self.get_success_url())
 
     def get_context_data(self, **kwargs):
@@ -685,7 +484,7 @@ email = EmailView.as_view()
 
 @method_decorator(login_required, name="dispatch")
 @method_decorator(rate_limit(action="change_password"), name="dispatch")
-class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
+class PasswordChangeView(AjaxCapableProcessFormViewMixin, NextRedirectMixin, FormView):
     template_name = "account/password_change." + app_settings.TEMPLATE_EXTENSION
     form_class = ChangePasswordForm
 
@@ -703,28 +502,12 @@ class PasswordChangeView(AjaxCapableProcessFormViewMixin, FormView):
         kwargs["user"] = self.request.user
         return kwargs
 
-    def get_success_url(self):
-        if self.success_url:
-            return self.success_url
+    def get_default_success_url(self):
         return get_adapter().get_password_change_redirect_url(self.request)
 
     def form_valid(self, form):
         form.save()
-        logout_on_password_change(self.request, form.user)
-        adapter = get_adapter(self.request)
-        adapter.add_message(
-            self.request,
-            messages.SUCCESS,
-            "account/messages/password_changed.txt",
-        )
-        adapter.send_notification_mail(
-            "account/email/password_changed", self.request.user
-        )
-        signals.password_changed.send(
-            sender=self.request.user.__class__,
-            request=self.request,
-            user=self.request.user,
-        )
+        flows.password_change.finalize_password_change(self.request, form.user)
         return super().form_valid(form)
 
     def get_context_data(self, **kwargs):
@@ -745,7 +528,7 @@ password_change = PasswordChangeView.as_view()
     rate_limit(action="change_password"),
     name="dispatch",
 )
-class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
+class PasswordSetView(AjaxCapableProcessFormViewMixin, NextRedirectMixin, FormView):
     template_name = "account/password_set." + app_settings.TEMPLATE_EXTENSION
     form_class = SetPasswordForm
 
@@ -763,25 +546,12 @@ class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
         kwargs["user"] = self.request.user
         return kwargs
 
-    def get_success_url(self):
-        if self.success_url:
-            return self.success_url
+    def get_default_success_url(self):
         return get_adapter().get_password_change_redirect_url(self.request)
 
     def form_valid(self, form):
         form.save()
-        user = form.user
-        logout_on_password_change(self.request, user)
-        adapter = get_adapter(self.request)
-        adapter.add_message(
-            self.request, messages.SUCCESS, "account/messages/password_set.txt"
-        )
-        signals.password_set.send(
-            sender=user.__class__,
-            request=self.request,
-            user=user,
-        )
-        adapter.send_notification_mail("account/email/password_set", user)
+        flows.password_change.finalize_password_set(self.request, form.user)
         return super().form_valid(form)
 
     def get_context_data(self, **kwargs):
@@ -795,11 +565,10 @@ class PasswordSetView(AjaxCapableProcessFormViewMixin, FormView):
 password_set = PasswordSetView.as_view()
 
 
-class PasswordResetView(AjaxCapableProcessFormViewMixin, FormView):
+class PasswordResetView(NextRedirectMixin, AjaxCapableProcessFormViewMixin, FormView):
     template_name = "account/password_reset." + app_settings.TEMPLATE_EXTENSION
     form_class = ResetPasswordForm
     success_url = reverse_lazy("account_reset_password_done")
-    redirect_field_name = REDIRECT_FIELD_NAME
 
     def get_form_class(self):
         return get_form_class(app_settings.FORMS, "reset_password", self.form_class)
@@ -813,13 +582,11 @@ class PasswordResetView(AjaxCapableProcessFormViewMixin, FormView):
         if r429:
             return r429
         form.save(self.request)
-        return super(PasswordResetView, self).form_valid(form)
+        return super().form_valid(form)
 
     def get_context_data(self, **kwargs):
-        ret = super(PasswordResetView, self).get_context_data(**kwargs)
-        login_url = passthrough_next_redirect_url(
-            self.request, reverse("account_login"), self.redirect_field_name
-        )
+        ret = super().get_context_data(**kwargs)
+        login_url = self.passthrough_next_url(reverse("account_login"))
         # NOTE: For backwards compatibility
         ret["password_reset_form"] = ret.get("form")
         # (end NOTE)
@@ -839,7 +606,10 @@ password_reset_done = PasswordResetDoneView.as_view()
 
 @method_decorator(rate_limit(action="reset_password_from_key"), name="dispatch")
 class PasswordResetFromKeyView(
-    AjaxCapableProcessFormViewMixin, LogoutFunctionalityMixin, FormView
+    AjaxCapableProcessFormViewMixin,
+    NextRedirectMixin,
+    LogoutFunctionalityMixin,
+    FormView,
 ):
     template_name = "account/password_reset_from_key." + app_settings.TEMPLATE_EXTENSION
     form_class = ResetPasswordKeyForm
@@ -877,9 +647,7 @@ class PasswordResetFromKeyView(
                     self.logout()
                     self.request.session[INTERNAL_RESET_SESSION_KEY] = self.key
 
-                return super(PasswordResetFromKeyView, self).dispatch(
-                    request, uidb36, self.key, **kwargs
-                )
+                return super().dispatch(request, uidb36, self.key, **kwargs)
         else:
             token_form = user_token_form_class(data={"uidb36": uidb36, "key": self.key})
             if token_form.is_valid():
@@ -888,7 +656,9 @@ class PasswordResetFromKeyView(
                 # avoids the possibility of leaking the key in the
                 # HTTP Referer header.
                 self.request.session[INTERNAL_RESET_SESSION_KEY] = self.key
-                redirect_url = self.request.path.replace(self.key, self.reset_url_key)
+                redirect_url = self.passthrough_next_url(
+                    self.request.path.replace(self.key, self.reset_url_key)
+                )
                 return redirect(redirect_url)
 
         self.reset_user = None
@@ -914,35 +684,12 @@ class PasswordResetFromKeyView(
 
     def form_valid(self, form):
         form.save()
-        adapter = get_adapter(self.request)
-
-        if self.reset_user:
-            # User successfully reset the password, clear any
-            # possible cache entries for all email addresses.
-            for email in self.reset_user.emailaddress_set.all():
-                adapter._delete_login_attempts_cached_email(
-                    self.request, email=email.email
-                )
-
-        adapter.add_message(
-            self.request,
-            messages.SUCCESS,
-            "account/messages/password_changed.txt",
-        )
-        signals.password_reset.send(
-            sender=self.reset_user.__class__,
-            request=self.request,
-            user=self.reset_user,
-        )
-        adapter.send_notification_mail("account/email/password_reset", self.reset_user)
-
+        flows.password_reset.finalize_password_reset(self.request, self.reset_user)
         if app_settings.LOGIN_ON_PASSWORD_RESET:
             return perform_login(
                 self.request,
                 self.reset_user,
-                email_verification=app_settings.EMAIL_VERIFICATION,
             )
-
         return super(PasswordResetFromKeyView, self).form_valid(form)
 
 
@@ -958,9 +705,8 @@ class PasswordResetFromKeyDoneView(TemplateView):
 password_reset_from_key_done = PasswordResetFromKeyDoneView.as_view()
 
 
-class LogoutView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
+class LogoutView(NextRedirectMixin, LogoutFunctionalityMixin, TemplateView):
     template_name = "account/logout." + app_settings.TEMPLATE_EXTENSION
-    redirect_field_name = REDIRECT_FIELD_NAME
 
     def get(self, *args, **kwargs):
         if app_settings.LOGOUT_ON_GET:
@@ -979,21 +725,10 @@ class LogoutView(TemplateResponseMixin, LogoutFunctionalityMixin, View):
         response = redirect(url)
         return _ajax_response(self.request, response)
 
-    def get_context_data(self, **kwargs):
-        ctx = kwargs
-        redirect_field_value = get_request_param(self.request, self.redirect_field_name)
-        ctx.update(
-            {
-                "redirect_field_name": self.redirect_field_name,
-                "redirect_field_value": redirect_field_value,
-            }
-        )
-        return ctx
-
     def get_redirect_url(self):
-        return get_next_redirect_url(
-            self.request, self.redirect_field_name
-        ) or get_adapter(self.request).get_logout_redirect_url(self.request)
+        return self.get_next_url() or get_adapter(self.request).get_logout_redirect_url(
+            self.request
+        )
 
 
 logout = LogoutView.as_view()
@@ -1013,9 +748,7 @@ class EmailVerificationSentView(TemplateView):
 email_verification_sent = EmailVerificationSentView.as_view()
 
 
-class BaseReauthenticateView(FormView):
-    redirect_field_name = REDIRECT_FIELD_NAME
-
+class BaseReauthenticateView(NextRedirectMixin, FormView):
     def dispatch(self, request, *args, **kwargs):
         resp = self._check_reauthentication_method_available(request)
         if resp:
@@ -1040,19 +773,14 @@ class BaseReauthenticateView(FormView):
         if not methods:
             # Reauthentication not available
             raise PermissionDenied("Reauthentication not available")
-        url = passthrough_next_redirect_url(
-            request, methods[0]["url"], self.redirect_field_name
-        )
+        url = self.passthrough_next_url(methods[0]["url"])
         return HttpResponseRedirect(url)
 
-    def get_success_url(self):
-        url = get_next_redirect_url(self.request, self.redirect_field_name)
-        if not url:
-            url = get_adapter(self.request).get_login_redirect_url(self.request)
+    def get_default_success_url(self):
+        url = get_adapter(self.request).get_login_redirect_url(self.request)
         return url
 
     def form_valid(self, form):
-        record_authentication(self.request, self.request.user)
         response = resume_request(self.request)
         if response:
             return response
@@ -1060,11 +788,8 @@ class BaseReauthenticateView(FormView):
 
     def get_context_data(self, **kwargs):
         ret = super().get_context_data(**kwargs)
-        redirect_field_value = get_request_param(self.request, self.redirect_field_name)
         ret.update(
             {
-                "redirect_field_name": self.redirect_field_name,
-                "redirect_field_value": redirect_field_value,
                 "reauthentication_alternatives": self.get_reauthentication_alternatives(),
             }
         )
@@ -1077,9 +802,7 @@ class BaseReauthenticateView(FormView):
             alt = dict(method)
             if self.request.path == alt["url"]:
                 continue
-            alt["url"] = passthrough_next_redirect_url(
-                self.request, alt["url"], self.redirect_field_name
-            )
+            alt["url"] = self.passthrough_next_url(alt["url"])
             alts.append(alt)
         alts = sorted(alts, key=lambda alt: alt["description"])
         return alts
@@ -1098,5 +821,97 @@ class ReauthenticateView(BaseReauthenticateView):
         ret["user"] = self.request.user
         return ret
 
+    def form_valid(self, form):
+        flows.reauthentication.reauthenticate_by_password(self.request)
+        return super().form_valid(form)
+
 
 reauthenticate = ReauthenticateView.as_view()
+
+
+class RequestLoginCodeView(RedirectAuthenticatedUserMixin, NextRedirectMixin, FormView):
+    form_class = RequestLoginCodeForm
+    template_name = "account/request_login_code." + app_settings.TEMPLATE_EXTENSION
+
+    @method_decorator(never_cache)
+    def dispatch(self, request, *args, **kwargs):
+        return super().dispatch(request, *args, **kwargs)
+
+    def get_form_class(self):
+        return get_form_class(app_settings.FORMS, "request_login_code", self.form_class)
+
+    def form_valid(self, form):
+        flows.login_by_code.request_login_code(self.request, form.cleaned_data["email"])
+        return super().form_valid(form)
+
+    def get_success_url(self):
+        if self.request.user.is_authenticated:
+            return None
+        url = reverse_lazy("account_confirm_login_code")
+        url = self.passthrough_next_url(reverse("account_confirm_login_code"))
+        return url
+
+    def get_context_data(self, **kwargs):
+        ret = super().get_context_data(**kwargs)
+        site = get_current_site(self.request)
+        ret.update({"site": site})
+        return ret
+
+
+request_login_code = RequestLoginCodeView.as_view()
+
+
+class ConfirmLoginCodeView(RedirectAuthenticatedUserMixin, NextRedirectMixin, FormView):
+    form_class = ConfirmLoginCodeForm
+    template_name = "account/confirm_login_code." + app_settings.TEMPLATE_EXTENSION
+
+    @method_decorator(never_cache)
+    def dispatch(self, request, *args, **kwargs):
+        self.user, self.pending_login = flows.login_by_code.get_pending_login(
+            request, peek=True
+        )
+        if not self.pending_login:
+            return HttpResponseRedirect(reverse("account_request_login_code"))
+        return super().dispatch(request, *args, **kwargs)
+
+    def get_form_class(self):
+        return get_form_class(app_settings.FORMS, "confirm_login_code", self.form_class)
+
+    def get_form_kwargs(self):
+        kwargs = super().get_form_kwargs()
+        kwargs["code"] = self.pending_login.get("code", "")
+        return kwargs
+
+    def form_valid(self, form):
+        redirect_url = self.get_next_url()
+        return flows.login_by_code.perform_login_by_code(
+            self.request, self.user, redirect_url, self.pending_login
+        )
+
+    def form_invalid(self, form):
+        attempts_left = flows.login_by_code.record_invalid_attempt(
+            self.request, self.pending_login
+        )
+        if attempts_left:
+            return super().form_invalid(form)
+        adapter = get_adapter(self.request)
+        adapter.add_message(
+            self.request,
+            messages.ERROR,
+            message=adapter.error_messages["too_many_login_attempts"],
+        )
+        return HttpResponseRedirect(reverse("account_request_login_code"))
+
+    def get_context_data(self, **kwargs):
+        ret = super().get_context_data(**kwargs)
+        site = get_current_site(self.request)
+        ret.update(
+            {
+                "site": site,
+                "email": self.pending_login["email"],
+            }
+        )
+        return ret
+
+
+confirm_login_code = ConfirmLoginCodeView.as_view()
