@@ -1,5 +1,6 @@
 from allauth.socialaccount.providers.base import AuthAction, ProviderAccount
 from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider
+from allauth.socialaccount.providers.ynab.views import YNABOAuth2Adapter
 
 
 class Scope(object):
@@ -14,13 +15,14 @@ class YNABProvider(OAuth2Provider):
     id = "ynab"
     name = "YNAB"
     account_class = YNABAccount
+    oauth2_adapter_class = YNABOAuth2Adapter
 
     def get_default_scope(self):
         scope = [Scope.ACCESS]
         return scope
 
-    def get_auth_params(self, request, action):
-        ret = super(YNABProvider, self).get_auth_params(request, action)
+    def get_auth_params_from_request(self, request, action):
+        ret = super().get_auth_params_from_request(request, action)
         if action == AuthAction.REAUTHENTICATE:
             ret["prompt"] = "select_account consent"
         return ret
